package com.mycompany.commerce.project.rest.resources;

import java.io.IOException;

import javax.servlet.ServletContext;
import javax.ws.rs.Encoded;
import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.QueryParam;
import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.HttpHeaders;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.UriInfo;

import org.apache.wink.common.model.atom.AtomFeed;

import com.ibm.commerce.foundation.client.util.oagis.SelectionCriteriaHelper;
import com.ibm.commerce.foundation.common.datatypes.BusinessContextType;
import com.ibm.commerce.foundation.rest.providers.AtomFeedSDOProvider;
import com.ibm.commerce.foundation.rest.util.StoreConfigurationHelper;
import com.mycompany.commerce.project.facade.client.ProjectFacadeClient;
import com.mycompany.commerce.project.facade.client.ProjectCollectionException;
import com.ibm.commerce.oagis9.datatypes.ExpressionType;
import com.ibm.commerce.oagis9.datatypes.GetType;
import com.ibm.icu.text.MessageFormat;
import commonj.sdo.DataObject;

/**
 * This class converts RESTful ProjectCollection requests into WebSphere
 * Commerce ProjectCollection requests and returns data in either XML, JSON, or
 * Atom feed format.
 */
@Path("stores/{storeId}/ProjectCollections")
@Encoded
public class ProjectCollection {

	private static final String CLASSNAME = ProjectCollection.class.getName();
	private static final java.util.logging.Logger LOGGER = com.ibm.commerce.foundation.common.util.logging.LoggingHelper
			.getLogger(ProjectCollection.class);
	
	/**
	 * The default access profile, used when an access profile is not specified using the <code>accessProfile</code> query parameter.
	 */
	protected static final String DEFAULT_ACCESS_PROFILE = "MyCompany_Store_Details";
	
	/**
	 * The default expression, used when a noun identifier is not provided.
	 */
	protected static final String DEFAULT_EXPRESSION = "/ProjectCollection[search(ends-with(ProjectCollectionIdentifier/ExternalIdentifier/Name, ''))]";
		
	/**
	 * The XPath key to find a ProjectCollection noun by projectcollectionidentifier.
	 */
	protected static final String FIND_BY_PROJECTCOLLECTIONIDENTIFIER_XPATH = "/ProjectCollection[ProjectCollectionIdentifier[(UniqueID={0})]]";
	/**
	 * The business context of the request.
	 */
	protected @Context BusinessContextType businessContext;
	
	/**
	 * The URI of the atom feed request.
	 */
	protected @Context UriInfo uri;
			
	/**
	 * The servlet context.
	 */
	protected @Context ServletContext context;
	
	/**
	 * The Http headers of the request.
	 */
	protected @Context HttpHeaders headers;

	/**
	 * This method performs a <code>GetProject</code> WebSphere Commerce
	 * service request based on the DEFAULT_EXPRESSION parameter.  This method is intended to return
	 * a collection of ProjectCollection resources.
	 * 
	 * @param storeId the store Id of the request.	 
	 * @param accessProfile the access profile.
	 * @param responseFormat the response format (xml or json).
	 * @return a collection of ProjectCollection nouns in the specified format.
	 * @throws WebApplicationException 
	 * @throws IOException
	 * @throws ProjectCollectionException
	 */
	@GET
	@Produces( { "application/atom+xml", "application/json", "application/xml" })
	public Response findProjectCollection(			
			@PathParam("storeId") String storeId,			
			@QueryParam(value = "accessProfile") String accessProfile,
			@QueryParam(value = "responseFormat") String responseFormat
			) throws WebApplicationException, IOException, ProjectCollectionException {

		final String METHODNAME = "findProjectCollection(String, String, String, String, String)";
		if (com.ibm.commerce.foundation.common.util.logging.LoggingHelper
				.isEntryExitTraceEnabled(LOGGER)) {
			LOGGER.entering(CLASSNAME, METHODNAME);
		}
														
		Response result = null;
		 // Perform the service request and return the response in the appropriate format.
		result = getProjectCollection(DEFAULT_EXPRESSION, accessProfile, responseFormat);
	   					
		if (com.ibm.commerce.foundation.common.util.logging.LoggingHelper
				.isEntryExitTraceEnabled(LOGGER)) {
			LOGGER.exiting(CLASSNAME, METHODNAME);
		}

		return result;

	}

	/**
	 * This method performs a GetProjectCollection by projectcollectionidentifier request.
	 * 
	 * @param projectcollectionidentifier the ProjectCollection projectcollectionidentifier.
	 * @param storeId the store Id.
	 * @param accessProfile the access profile of the Get request.
	 * @param responseFormat the response format (xml, json, or atom).
	 * @return The ProjectCollection noun in the specified format.
	 * @throws WebApplicationException 
	 * @throws IOException
	 * @throws ProjectCollectionException
	 */
	@GET
	@Produces( { "application/atom+xml", "application/json", "application/xml" })
	@Path("{projectcollectionidentifier}")
	public Response findByProjectCollectionIdentifier(
	        @PathParam("projectcollectionidentifier") String projectcollectionidentifier,
	        @PathParam("storeId") String storeId,
	        @QueryParam(value = "accessProfile") String accessProfile,
	        @QueryParam(value = "responseFormat") String responseFormat
	        ) throws WebApplicationException, IOException, ProjectCollectionException {
	
	    final String METHODNAME = "findByProjectCollectionIdentifier(String, String, String, String, String, String)";
	
	    if (com.ibm.commerce.foundation.common.util.logging.LoggingHelper
	            .isEntryExitTraceEnabled(LOGGER)) {
	        LOGGER.entering(CLASSNAME, METHODNAME);
	    }
	    	  	
	    Response result = null;
	    Object[] params = { projectcollectionidentifier };
	    String expression = MessageFormat.format(FIND_BY_PROJECTCOLLECTIONIDENTIFIER_XPATH, params);
	
	    // Perform the service request and return the response in the appropriate format.
	    result = getProjectCollection(expression, accessProfile, responseFormat);
	    	
	    if (com.ibm.commerce.foundation.common.util.logging.LoggingHelper
	            .isEntryExitTraceEnabled(LOGGER)) {
	        LOGGER.exiting(CLASSNAME, METHODNAME);
	    }
	
	    return result;
	
	}

	/**
	 * This method performs a <code>GetProjectCollection</code> service request.
	 * 
	 * @param expression the XPath expression.
	 * @param accessProfile the access profile of the get expression.
	 * @param responseFormat the response format.
	 * @return the ProjectCollection noun.
	 * @throws ProjectCollectionException the exception representing the error of the
	 *             <code>GetProjectCollection<code> request.
	 * @throws WebApplicationException 
	 * @throws IOException
	 */
	protected Response getProjectCollection(String expression, String accessProfile, String responseFormat)
			throws ProjectCollectionException, WebApplicationException, IOException {

		final String METHODNAME = "getProjectCollection(String, String, String)";

		if (com.ibm.commerce.foundation.common.util.logging.LoggingHelper
				.isEntryExitTraceEnabled(LOGGER)) {
			LOGGER.entering(CLASSNAME, METHODNAME);
		}
		
		 // Get the response format, either from the Http header or URL, the URL takes precedence.
	    MediaType format = StoreConfigurationHelper.getResponseFormat(responseFormat, headers);
		
		if (accessProfile == null) {
	    	accessProfile = DEFAULT_ACCESS_PROFILE;
	    }
						
		// Build the Get request.
		SelectionCriteriaHelper selectionCriteria = buildExpression(expression, accessProfile);
		GetType getVerb = buildGetVerb(selectionCriteria.getSelectionCriteriaExpression());
						
		// Perform the service request.
		ProjectFacadeClient client = new ProjectFacadeClient(businessContext, null);
		DataObject dataArea = (DataObject) client.getProjectCollection(getVerb);
																
		// For XML or JSON, the JAX-RS runtime will resolve the appropriate provider.
		// For Atom, the feed is directly populated.
		Response result = null;
		if (MediaType.APPLICATION_ATOM_XML_TYPE.isCompatible(format)) {
			AtomFeedSDOProvider provider = new AtomFeedSDOProvider();
			AtomFeed feed = provider.writeTo(dataArea, businessContext, uri, context);
			result = Response.ok(feed).type(MediaType.APPLICATION_ATOM_XML).build();
		} else {
			result = Response.ok(dataArea).type(format).build();
		}	
		
		if (com.ibm.commerce.foundation.common.util.logging.LoggingHelper
				.isEntryExitTraceEnabled(LOGGER)) {
			LOGGER.exiting(CLASSNAME, METHODNAME);
		}

		return result;
	}	
	
	/**
	 * This method creates the Project Get Verb.  This method can be 
	 * overridden to add paging information.
	 * @param expression The get expression.
	 * @return The Get Verb.
	 */
	protected GetType buildGetVerb(ExpressionType expression) {		
		return ProjectFacadeClient.createGetVerb(expression);
	}
	
	/**
	 * This method creates a <code>SelectionCriteriaHelper</code> object with the provided XPath expression and
	 * access profile.  This method can be overridden to add additional control parameters to the expression. 
	 * @param expression the XPath expression of the request.
	 * @param accessProfile the access profile of the request.
	 * @return the <code>SelectionCriteriaHelper</code> with the provided XPath expression and access profile.
	 */
	protected SelectionCriteriaHelper buildExpression(String expression, String accessProfile) {
		SelectionCriteriaHelper selectionCriteria = new SelectionCriteriaHelper(expression);
		selectionCriteria.addAccessProfile(accessProfile);		
		return selectionCriteria;
	}
	
}